@@ -1,8 +1,8 @@
-// Copyright (C) 2004, 2008 International Business Machines and others.
+// Copyright (C) 2004, 2009 International Business Machines and others.
 // All Rights Reserved.
 // This code is published under the Common Public License.
 //
-// $Id: IpIpoptData.hpp 1210 2008-04-05 03:01:56Z andreasw $
+// $Id: IpIpoptData.hpp 1435 2009-04-24 17:09:05Z andreasw $
 //
 // Authors:  Carl Laird, Andreas Waechter     IBM    2004-08-13
 
@@ -20,7 +20,56 @@ namespace Ipopt
 
   /* Forward declaration */
   class IpoptNLP;
-  class CGPenaltyData;
+
+  /** Base class for additional data that is special to a particular
+   *  type of algorithm, such as the CG penalty function, or using
+   *  iterative linear solvers.  The regular IpoptData object should
+   *  be given a derivation of this base class when it is created. */
+  class IpoptAdditionalData : public ReferencedObject
+  {
+  public:
+    /**@name Constructors/Destructors */
+    //@{
+    /** Default Constructor */
+    IpoptAdditionalData()
+    {}
+
+    /** Default destructor */
+    virtual ~IpoptAdditionalData()
+    {}
+    //@}
+
+    /** This method is called to initialize the global algorithmic
+     *  parameters.  The parameters are taken from the OptionsList
+     *  object. */
+    virtual bool Initialize(const Journalist& jnlst,
+                            const OptionsList& options,
+                            const std::string& prefix) = 0;
+
+    /** Initialize Data Structures at the beginning. */
+    virtual bool InitializeDataStructures() = 0;
+
+    /** Do whatever is necessary to accept a trial point as current
+     *  iterate.  This is also used to finish an iteration, i.e., to
+     *  release memory, and to reset any flags for a new iteration. */
+    virtual void AcceptTrialPoint() = 0;
+
+  private:
+    /**@name Default Compiler Generated Methods
+     * (Hidden to avoid implicit creation/calling).
+     * These methods are not implemented and 
+     * we do not want the compiler to implement
+     * them for us, so we declare them private
+     * and do not define them. This ensures that
+     * they will not be implicitly created/called. */
+    //@{
+    /** Copy Constructor */
+    IpoptAdditionalData(const IpoptAdditionalData&);
+
+    /** Overloaded Equals Operator */
+    void operator=(const IpoptAdditionalData&);
+    //@}
+  };
 
   /** Class to organize all the data required by the algorithm.
    *  Internally, once this Data object has been initialized, all
@@ -37,10 +86,11 @@ namespace Ipopt
     /**@name Constructors/Destructors */
     //@{
     /** Constructor */
-    IpoptData();
+    IpoptData(SmartPtr<IpoptAdditionalData> add_data = NULL,
+              Number cpu_time_start = -1.);
 
     /** Default destructor */
-    ~IpoptData();
+    virtual ~IpoptData();
     //@}
 
     /** Initialize Data Structures */
@@ -308,6 +358,14 @@ namespace Ipopt
     }
     //@}
 
+    /** Cpu time counter at the beginning of the optimization.  This
+     *  is useful to see how much CPU time has been spent in this
+     *  optimization run. */
+    Number cpu_time_start() const
+    {
+      return cpu_time_start_;
+    }
+
     /** @name Information gathered for iteration output */
     //@{
     Number info_regu_x() const
@@ -388,10 +446,23 @@ namespace Ipopt
       return timing_statistics_;
     }
 
-    /** Get access to the Chen-Goldbarb penalty method specific data */
-    CGPenaltyData& CGPenData()
+    /** Check if additional data has been set */
+    bool HaveAddData()
     {
-      return *cgpen_data_;
+      return IsValid(add_data_);
+    }
+
+    /** Get access to additional data object */
+    IpoptAdditionalData& AdditionalData()
+    {
+      return *add_data_;
+    }
+
+    /** Set a new pointer for additional Ipopt data */
+    void SetAddData(SmartPtr<IpoptAdditionalData> add_data)
+    {
+      DBG_ASSERT(!HaveAddData());
+      add_data_ = add_data;
     }
 
     /** Set the perturbation of the primal-dual system */
@@ -523,9 +594,12 @@ namespace Ipopt
      *  statistics */
     TimingStatistics timing_statistics_;
 
+    /** CPU time counter at initialization. */
+    Number cpu_time_start_;
+
     /** Object for the data specific for the Chen-Goldfarb penalty
      *  method algorithm */
-    CGPenaltyData* const cgpen_data_;
+    SmartPtr<IpoptAdditionalData> add_data_;
 
     /** @name Information about the perturbation of the primal-dual
      *  system */
