@@ -1,4 +1,4 @@
-// Copyright (C) 2006, 2007 Damien Hocking, KBC Advanced Technologies
+// Copyright (C) 2006, 2009 Damien Hocking, KBC Advanced Technologies
 // All Rights Reserved.
 // This code is published under the Common Public License.
 //
@@ -10,17 +10,20 @@
 //           (major revision)
 //           (incorporated by AW on 2006-11-11 into Ipopt package)
 
-#include "IpMumpsSolverInterface.hpp"
-
+// The first header to include is the one for MPI.  We need to make sure
+// that we are using C calling mangling
 extern "C"
 {
-#include "dmumps_c.h"
   // The following line is a fix for otherwise twice-defined global variable
   // (This would have to be taken out for a parallel MUMPS version!)
-#define MPI_COMM_WORLD IPOPT_MPI_COMM_WORLD
-#include "mpi.h"
+# define MPI_COMM_WORLD IPOPT_MPI_COMM_WORLD
+# include "mpi.h"
 }
 
+#include "IpMumpsSolverInterface.hpp"
+
+#include "dmumps_c.h"
+
 #ifdef HAVE_CMATH
 # include <cmath>
 #else
@@ -331,7 +334,11 @@ namespace Ipopt
 
     dump_matrix(mumps_data);
 
+    Jnlst().Printf(J_MOREDETAILED, J_LINEAR_ALGEBRA,
+                   "Calling MUMPS-1 for symbolic factorization at cpu time %10.3f (wall %10.3f).\n", CpuTime(), WallclockTime());
     dmumps_c(mumps_data);
+    Jnlst().Printf(J_MOREDETAILED, J_LINEAR_ALGEBRA,
+                   "Done with MUMPS-1 for symbolic factorization at cpu time %10.3f (wall %10.3f).\n", CpuTime(), WallclockTime());
     int error = mumps_data->info[0];
     const int& mumps_permuting_scaling_used = mumps_data->infog[22];
     const int& mumps_pivot_order_used = mumps_data->infog[6];
@@ -342,6 +349,10 @@ namespace Ipopt
                    "           scaling will be %d.\n",
                    mumps_data->icntl[7]);
 
+    if (HaveIpData()) {
+      IpData().TimingStats().LinearSystemSymbolicFactorization().End();
+    }
+
     //return appropriat value
     if (error == -6) {//system is singular
       Jnlst().Printf(J_DETAILED, J_LINEAR_ALGEBRA,
@@ -355,10 +366,6 @@ namespace Ipopt
       return SYMSOLVER_FATAL_ERROR;
     }
 
-    if (HaveIpData()) {
-      IpData().TimingStats().LinearSystemSymbolicFactorization().End();
-    }
-
     return SYMSOLVER_SUCCESS;
   }
 
@@ -371,7 +378,11 @@ namespace Ipopt
     mumps_data->job = 2;//numerical factorization
 
     dump_matrix(mumps_data);
+    Jnlst().Printf(J_MOREDETAILED, J_LINEAR_ALGEBRA,
+                   "Calling MUMPS-2 for numerical factorization at cpu time %10.3f (wall %10.3f).\n", CpuTime(), WallclockTime());
     dmumps_c(mumps_data);
+    Jnlst().Printf(J_MOREDETAILED, J_LINEAR_ALGEBRA,
+                   "Done with MUMPS-2 for numerical factorization at cpu time %10.3f (wall %10.3f).\n", CpuTime(), WallclockTime());
     int error = mumps_data->info[0];
 
     //Check for errors
@@ -388,7 +399,11 @@ namespace Ipopt
         Jnlst().Printf(J_WARNING, J_LINEAR_ALGEBRA, "%d.\n", mumps_data->icntl[13]);
 
         dump_matrix(mumps_data);
+        Jnlst().Printf(J_MOREDETAILED, J_LINEAR_ALGEBRA,
+                       "Calling MUMPS-2 (repeated) for numerical factorization at cpu time %10.3f (wall %10.3f).\n", CpuTime(), WallclockTime());
         dmumps_c(mumps_data);
+        Jnlst().Printf(J_MOREDETAILED, J_LINEAR_ALGEBRA,
+                       "Done with MUMPS-2 (repeated) for numerical factorization at cpu time %10.3f (wall %10.3f).\n", CpuTime(), WallclockTime());
         error = mumps_data->info[0];
         if (error != -8 && error != -9)
           break;
@@ -415,16 +430,9 @@ namespace Ipopt
 
     negevals_ = mumps_data->infog[11];
 
-    if (check_NegEVals && (numberOfNegEVals!=negevals_)) {
-      Jnlst().Printf(J_DETAILED, J_LINEAR_ALGEBRA,
-                     "In MumpsSolverInterface::Factorization: negevals_ = %d, but numberOfNegEVals = %d\n",
-                     negevals_, numberOfNegEVals);
-      return SYMSOLVER_WRONG_INERTIA;
-    }
-
     if (error == -13) {
       Jnlst().Printf(J_ERROR, J_LINEAR_ALGEBRA,
-                     "MUMPS returned INFO(1) =%d - out or memory.\n",
+                     "MUMPS returned INFO(1) =%d - out or memory.\nIn some cases it helps to decrease the value of the option \"mumps_mem_percent\".\n",
                      error);
       return SYMSOLVER_FATAL_ERROR;
     }
@@ -435,6 +443,13 @@ namespace Ipopt
       return SYMSOLVER_FATAL_ERROR;
     }
 
+    if (check_NegEVals && (numberOfNegEVals!=negevals_)) {
+      Jnlst().Printf(J_DETAILED, J_LINEAR_ALGEBRA,
+                     "In MumpsSolverInterface::Factorization: negevals_ = %d, but numberOfNegEVals = %d\n",
+                     negevals_, numberOfNegEVals);
+      return SYMSOLVER_WRONG_INERTIA;
+    }
+
     return SYMSOLVER_SUCCESS;
   }
 
@@ -450,7 +465,11 @@ namespace Ipopt
       Index offset = i * mumps_data->n;
       mumps_data->rhs = &(rhs_vals[offset]);
       mumps_data->job = 3;//solve
+      Jnlst().Printf(J_MOREDETAILED, J_LINEAR_ALGEBRA,
+                     "Calling MUMPS-3 for solve at cpu time %10.3f (wall %10.3f).\n", CpuTime(), WallclockTime());
       dmumps_c(mumps_data);
+      Jnlst().Printf(J_MOREDETAILED, J_LINEAR_ALGEBRA,
+                     "Done with MUMPS-3 for solve at cpu time %10.3f (wall %10.3f).\n", CpuTime(), WallclockTime());
       int error = mumps_data->info[0];
       if (error < 0) {
         Jnlst().Printf(J_ERROR, J_LINEAR_ALGEBRA,
