@@ -1,8 +1,8 @@
-// Copyright (C) 2004, 2006 International Business Machines and others.
+// Copyright (C) 2004, 2008 International Business Machines and others.
 // All Rights Reserved.
 // This code is published under the Common Public License.
 //
-// $Id: IpOptionsList.cpp 1154 2008-02-19 19:34:59Z andreasw $
+// $Id: IpOptionsList.cpp 1435 2009-04-24 17:09:05Z andreasw $
 //
 // Authors:  Carl Laird, Andreas Waechter     IBM    2004-08-13
 
@@ -39,6 +39,16 @@
 # endif
 #endif
 
+#ifdef HAVE_CSTRING
+# include <cstring>
+#else
+# ifdef HAVE_STRING_H
+#  include <string.h>
+# else
+#  error "don't have header file for stdlib"
+# endif
+#endif
+
 namespace Ipopt
 {
 
@@ -83,8 +93,8 @@ namespace Ipopt
 
       if (!option->IsValidStringSetting(value)) {
         if (IsValid(jnlst_)) {
-          std::string msg = "Setting: " + value;
-          msg += " is not a valid setting for Option: ";
+          std::string msg = "Setting: \"" + value;
+          msg += "\" is not a valid setting for Option: ";
           msg += tag;
           msg += ". Check the option documentation.\n";
           jnlst_->Printf(J_ERROR, J_MAIN, msg.c_str());
@@ -166,9 +176,9 @@ namespace Ipopt
 
       if (!option->IsValidNumberSetting(value)) {
         if (IsValid(jnlst_)) {
-          std::string msg = "Setting: ";
+          std::string msg = "Setting: \"";
           msg += buffer;
-          msg += " is not a valid setting for Option: ";
+          msg += "\" is not a valid setting for Option: ";
           msg += tag;
           msg += ". Check the option documentation.\n";
           jnlst_->Printf(J_ERROR, J_MAIN, msg.c_str());
@@ -241,9 +251,9 @@ namespace Ipopt
 
       if (!option->IsValidIntegerSetting(value)) {
         if (IsValid(jnlst_)) {
-          std::string msg = "Setting: ";
+          std::string msg = "Setting: \"";
           msg += buffer;
-          msg += " is not a valid setting for Option: ";
+          msg += "\" is not a valid setting for Option: ";
           msg += tag;
           msg += ". Check the option documentation.\n";
           jnlst_->Printf(J_ERROR, J_MAIN, msg.c_str());
@@ -457,14 +467,25 @@ namespace Ipopt
 
     std::string strvalue;
     if (find_tag(tag, prefix, strvalue)) {
+      // Some people like to use 'd' instead of 'e' in floating point
+      // numbers.  Therefore, we change a 'd' to an 'e'
+      char* buffer = new char[strvalue.length()+1];
+      strcpy(buffer, strvalue.c_str());
+      for (int i=0; i<(int)strvalue.length(); ++i) {
+        if (buffer[i]=='d' || buffer[i]=='D') {
+          buffer[i] = 'e';
+        }
+      }
       char* p_end;
-      Number retval = strtod(strvalue.c_str(), &p_end);
+      Number retval = strtod(buffer, &p_end);
       if (*p_end!='\0' && !isspace(*p_end)) {
+        delete [] buffer;
         std::string msg = "Option \"" + tag +
                           "\": Double value expected, but non-numeric value \"" +
                           strvalue+"\" found.\n";
         THROW_EXCEPTION(OPTION_INVALID, msg);
       }
+      delete [] buffer;
       value = retval;
       return true;
     }
@@ -611,9 +632,9 @@ namespace Ipopt
       if (IsValid(reg_options_)) {
         SmartPtr<const RegisteredOption> option = reg_options_->GetOption(tag);
         if (IsNull(option)) {
-          std::string msg = "Read Option: ";
+          std::string msg = "Read Option: \"";
           msg += tag;
-          msg += ". It is not a valid option. Check the list of available options.";
+          msg += "\". It is not a valid option. Check the list of available options.";
           THROW_EXCEPTION(OPTION_INVALID, msg);
         }
 
@@ -623,14 +644,25 @@ namespace Ipopt
                            "Error setting string value read from option file.");
         }
         else if (option->Type() == OT_Number) {
+          // Some people like to use 'd' instead of 'e' in floating
+          // point numbers.  Therefore, we change a 'd' to an 'e'
+          char* buffer = new char[value.length()+1];
+          strcpy(buffer, value.c_str());
+          for (int i=0; i<(int)value.length(); ++i) {
+            if (buffer[i]=='d' || buffer[i]=='D') {
+              buffer[i] = 'e';
+            }
+          }
           char* p_end;
-          Number retval = strtod(value.c_str(), &p_end);
+          Number retval = strtod(buffer, &p_end);
           if (*p_end!='\0' && !isspace(*p_end)) {
+            delete [] buffer;
             std::string msg = "Option \"" + tag +
                               "\": Double value expected, but non-numeric option value \"" +
                               value + "\" found.\n";
             THROW_EXCEPTION(OPTION_INVALID, msg);
           }
+          delete [] buffer;
           bool result = SetNumericValue(tag, retval, false);
           ASSERT_EXCEPTION(result, OPTION_INVALID,
                            "Error setting numeric value read from file.");
@@ -717,10 +749,21 @@ namespace Ipopt
       c=is.get();
     }
 
+    bool inside_quotes = (c=='"');
+    if (inside_quotes) {
+      if (is.eof()) return false; // eof after quotation symbol
+      c=is.get();
+    }
+
     // Now read the token
-    while (!is.eof() && !isspace(c)) {
+    while (!is.eof() && (inside_quotes || !isspace(c))) {
       token += c;
       c = is.get();
+      if (inside_quotes && (c=='"')) {
+        inside_quotes = false;
+        if (!is.eof())
+          c = is.get();
+      }
     }
 
     return (!is.eof());
